# -*- coding: utf-8 -*-

# Trac imports.
from genshi.builder import tag
from trac.core import *
from trac.wiki.macros import WikiMacroBase
from trac.util.datefmt import format_date

# DiscussionPlugin imports.
from tracdiscussion.api import *
from tracdiscussion.core import *

class RecentTopicsMacro(WikiMacroBase):
    """List all topics pages that have recently been active, 
      grouping them by the day they were last active.
      
      This macro accepts two parameters. The first is a forum id: 
      If provided, only topics in that forum are included in the resulting list. 
      If omitted, topics from all forums are listed. 
      
      The second parameter is a number for limiting the number of topics returned. 
      For example, specifying a limit of 5 will result in only the five most 
      recently active topics to be included in the list.
      """

    def expand_macro(self, formatter, name, content):
        # Check permission
        if not formatter.perm.has_permission('DISCUSSION_VIEW'):
            return

        # Get database access.
        cursor = formatter.db.cursor()
        
        # Create request context.
        context = Context.from_request(formatter.req)
        context.realm = 'discussion-wiki'
        context.cursor = cursor

        # Get API object.
        api = self.env[DiscussionApi]

        # Parse arguments
        specified_forum_id = limit = None
        if content:
            argv = [arg.strip() for arg in content.split(',')]
            if len(argv) > 0:
                specified_forum_id = argv[0]
                if len(argv) > 1:
                    limit = int(argv[1])

        # Construct and execute SQL query
        sql = """SELECT forum, topic, max(time) as max_time FROM (
            SELECT forum, topic, time FROM message
            UNION SELECT forum, id as topic, time FROM topic)
            """
        args = []
        if specified_forum_id:
            sql += " WHERE forum = %s"
            args.append(specified_forum_id)
        sql += " GROUP BY topic ORDER BY max_time DESC"
        if limit:
            sql += " LIMIT %s"
            args.append(limit)

        cursor.execute(sql, args)
        rows = [(forum_id, topic_id, time)
            for forum_id, topic_id, time in cursor]
            
        # Collect recent topics
        entries_per_date = []
        prevdate = None
        for forum_id, topic_id, time in rows:
            date = format_date(time)
            if date != prevdate:
                prevdate = date
                entries_per_date.append((date, []))
            forum_name = api.get_forum(context, forum_id)['name']
            topic_subject = api.get_topic_subject(context, topic_id)
            entries_per_date[-1][1].append((forum_id, forum_name, topic_id, topic_subject))
           
        # Format result
        return tag.div(
            (tag.h3(date),
             tag.ul(
                 tag.li(tag.a(forum_name, href=formatter.href.discussion('forum', forum_id)), ': ',
                        tag.a(topic_subject, href=formatter.href.discussion('topic', topic_id)))
                 for forum_id, forum_name, topic_id, topic_subject in entries))
            for date, entries in entries_per_date)
